import ObjetoDinamico from './ObjetoDinamico';
// eslint-disable-next-line import/no-cycle
import Rotulo from './Rotulo';

/**
 * Classe Ponto
 *
 * Utilizada para representar um Ponto no desenho geométrico
 *
 * @author Robertino Mendes Santiago Jr <robertino@ufpr.br>
 */
export default class Ponto extends ObjetoDinamico {
  /**
   * Método construtor da classe Ponto
   * @param {number} x Posição relativa ao eixo x onde o ponto será criado
   * @param {number} y Posição relativa ao eixo y onde o ponto será criado
   */
  constructor(x, y) {
    super();
    this.x = x;
    this.y = y;
    this.raio = 4;
    this.espessura = 1;
    this.selecionado = false;
    this.rotulo = new Rotulo(this.x, this.y, this);
    this.elemento = this.criaPonto();
  }

  /**
   * Método que devolve o elemento Ponto.
   * @return {Object} Retorna o elemento que representa um Ponto em formato SVG
   */
  draw() {
    return this.elemento;
  }

  /**
   * Verifica se existe um ponto na posição especificada por x e y
   * @param {number} x - Valor da posição no eixo x
   * @param {number} y - Valor da posição no eixo y
   * @return {boolean} Retorna verdadeiro se existe um ponto na posição especificada.
   * Caso contrário, retorna falso.
   */
  contains(x, y) {
    const xDiff = Math.abs(this.x - x);
    const yDiff = Math.abs(this.y - y);
    const dist = Math.sqrt((xDiff ** 2) + (yDiff ** 2));
    return dist <= this.raio;
  }

  /**
   * Método que cria um elemento SVG que representa um Ponto
   * @return {Object} Retorna o elemento SVG criado que representa o Ponto
   */
  criaPonto() {
    const agrupador = document.createElementNS(this.svgNS, 'g');

    const ponto = document.createElementNS(this.svgNS, 'circle');
    ponto.setAttribute('id', 'mycircle');
    ponto.setAttribute('cx', this.x);
    ponto.setAttribute('cy', this.y);
    ponto.setAttribute('r', this.raio);
    ponto.setAttribute('fill', 'lightgreen');
    ponto.setAttribute('stroke', 'black');
    ponto.setAttribute('stroke-width', this.espessura);
    agrupador.appendChild(ponto);

    const rotulo = this.rotulo.draw();
    agrupador.appendChild(rotulo);

    return agrupador;
  }

  /**
   * Método que realiza a movimentação do Ponto
   * @param {number} x Posição relativa ao eixo x onde o ponto será movimentado
   * @param {number} y Posição relativa ao eixo y onde o ponto será movimentado
   */
  mover(x, y) {
    this.setX(x);
    this.setY(y);
  }

  /**
   * Devolve o valor da propriedade x
   * @return {number} Valor da propriedade x
   */
  getX() {
    return this.x;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade x
   * @param {number} x Posição relativa ao eixo x
   */
  setX(x) {
    this.x = x;
    this.elemento.setAttribute('cx', x);
  }

  /**
   * Devolve o valor da propriedade y
   * @return {number} Valor da propriedade y
   */
  getY() {
    return this.y;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade y
   * @param {number} y Posição relativa ao eixo y
   */
  setY(y) {
    this.y = y;
    this.elemento.setAttribute('cy', y);
  }

  /**
   * Devolve o valor da propriedade raio
   * @return {number} Valor da propriedade raio
   */
  getRaio() {
    return this.raio;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade raio
   * @param {number} raio Valor do raio do Ponto
   */
  setRaio(raio) {
    this.raio = raio;
    this.elemento.setAttribute('r', raio);
  }

  /**
   * Devolve o valor da propriedade espessura da borda
   * @return {number} Valor da propriedade da espessura da borda
   */
  getEspessura() {
    return this.espessura;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade espessura
   * @param {number} espessura Valor da espessura da borda
   */
  setEspessura(espessura) {
    this.espessura = espessura;
    this.elemento.setAttribute('stroke-width', espessura);
  }

  /**
   * Devolve o valor da propriedade selecionado
   * @return {boolean} Valor da propriedade selecionado
   */
  getSelecionado() {
    return this.selecionado;
  }

  /**
   * Atribui um valor recebido por parâmetro à propriedade selecionado
   * @param {boolean} selecionado Verdadeiro se o objeto está selecionado ou falso caso contrário
   */
  setSelecionado(selecionado) {
    this.selecionado = selecionado;
    if (selecionado) {
      this.elemento.setAttribute('stroke', 'red');
    } else {
      this.elemento.setAttribute('stroke', 'black');
    }
  }
}
